/*******************************************************************************
* NXP Semiconductor Inc.
* (c) Copyright 2015 NXP Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by NXP in this matter are performed AS IS and without 
any warranty. CUSTOMER retains the final decision relative to the total design 
and functionality of the end product. NXP neither guarantees nor will be 
held liable by CUSTOMER for the success of this project.
NXP DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING, 
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR 
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED TO THE PROJECT
BY NXP, AND OR NAY PRODUCT RESULTING FROM NXP SERVICES. IN NO EVENT
SHALL NXP BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF 
THIS AGREEMENT.

CUSTOMER agrees to hold NXP harmless against any and all claims demands or 
actions by anyone on account of any damage, or injury, whether commercial, 
contractual, or tortuous, rising directly or indirectly as a result of the advise
or assistance supplied CUSTOMER in connection with product, services or goods 
supplied under this Agreement.
********************************************************************************
* File:             main.c
* Owner             Petr Stancik
* Version           1.0
* Date              Sep-15-2016
* Classification:   General Business Information
* Brief:	    MCAN FD TX/RX demo 
                    based on pin toggle stationery for MPC5777C
********************************************************************************
********************************************************************************
* Detailed Description:
*
* Configures the MCANs to transmit and receive a CAN FD message with or without
* bit rate switching for data phase. This is defined by BRS macro.
* Baudrate during arbitration phase is set to 500kbps, during data phase 1Mpbs
* because of PHY used on the EVB.
* 
* In this config, MCAN_0 transmits a message. MCAN_1 receives the message.
*
* MCAN_0 sends message each 1sec. This interval is generated by PIT.
* Single TX buffer is used to send n bytes. The message ID is changed for each
* transmission. Two standard and 2 extended IDs are sent.
*
* MCAN_1 is configured to receive a message, ISR is used to read new message.
* There are 2 standard and 2 extended ID filter tables defined. Classic filter
* configuration is set, means filter ID & mask.
* Messages with matched standard ID are received into RXFIFO_0, messages with matched
* extended ID then stored in RXFIFO_1.
*  
* EVB connection:
*
* J37 and J38 to position 2-3 to connect MCAN1 TX/RX to transceiver
* 
* CAN0-CANH on P15-1 to CAN1-CANH on P14-1
* CAN0-CANL on P15-2 to CAN1-CANL on P14-2
* 
* 
* ------------------------------------------------------------------------------
* Test HW:         MPC5777C-512DS Rev.A + MPC57xx MOTHER BOARD Rev.C
* MCU:             PPC5777CMM03 2N45H CTZZS1521A
* Fsys:            PLL1 = core_clk = 264MHz, PLL0 = 192MHz
* Debugger:        Lauterbach Trace32
* Target:          internal_FLASH
* Terminal:        19200-8-no parity-1 stop bit-no flow control on eSCI_A
*		   use USB connector (J21) on minimodule
*
* EVB connection:  ETPUA30 (PortP P23-15) --> USER_LED_1 (P7-1)
*                  ETPUA31 (PortP P23-14) --> USER_LED_2 (P7-2)
*
********************************************************************************
Revision History:
Ver  Date         Author            Description of Changes
0.1  Aug-26-2015  David Tosenovjan  Initial version
0.2  Sep-01-2015  David Tosenovjan  printf added, minor edits
0.3  Nov-26-2015  David Tosenovjan  - lauterbach scripts changed
                                    - multicore added
                                    - intc_sw_handlers.s changed
                                    - Interrupts.c changed
                                    - other edits
1.0  Sep-15-2016  Petr Stancik	    - Added MCAN related code				    
*******************************************************************************/
#include <ppc_ghs.h>

#include "MPC577xC.h"
#include "Interrupts.h"
#include "uart.h"
#include "stdio.h"

/*******************************************************************************
* External objects
*******************************************************************************/

/* Prototype for PRC1 startup */
#ifdef __cplusplus
    extern "C"{void __start();}
#else
    extern void __start();
#endif


/*******************************************************************************
* Global variables
*******************************************************************************/
uint8_t MCAN_1_data[64];
uint8_t MCAN_0_data[8]={'.','.','.','H','e','l','l','o'};

uint8_t tx_cnt1 = 0;

/*******************************************************************************
* Constants and macros
*******************************************************************************/
/* FlexCAN module register's bit masks */
#define MCAN0_BASE_ADDR 0xFFE30000
#define MCAN1_BASE_ADDR 0xFFEE8000

#define MCAN_MSG_RAM_BASE_ADDR 0xFFE34000 // base address of shared MSG RAM
#define MCAN_MSG_RAM_LENGTH 0x4000

//#define MCAN0_SIDFC_FLSSA 0x0000   	//STANDARD MESSAGE ID FILTERS
//#define MCAN0_XIDFC_FLESA 0x0200   	//EXTENDED MESSAGE ID FILTERS
//#define MCAN0_RXF0C_F0SA 0x0400    	//RXFIFO 0 ELEMENTS
//#define MCAN0_RXF1C_F1SA 0x0800	//RXFIFO 1 ELEMEMNTS
//#define MCAN0_RXF1S_DMS 0x0BF4  
//#define MCAN0_TXEFC_EFSA 0x0C00    	//TX Event FIFO
#define MCAN0_TXBC_TBSA 0x0000     	//TXBUFFER ELEMENTS

#define MCAN1_SIDFC_FLSSA 0x0080	//STANDARD MESSAGE ID FILTERS
#define MCAN1_XIDFC_FLESA 0x00C0	//EXTENDED MESSAGE ID FILTERS
#define MCAN1_RXF0C_F0SA 0x0100		//RXFIFO 0 ELEMENTS
#define MCAN1_RXF1C_F1SA 0x0180		//RXFIFO 1 ELEMEMNTS
//#define MCAN1_RXF1S_DMS 0x1AF4
//#define MCAN1_TXEFC_EFSA 0x1B00	//TX Event FIFO
//#define MCAN1_TXBC_TBSA 0x1C00	//TXBUFFER ELEMENTS

#define WRITE_MSG_RAM(offset, data) *(vuint32_t*)(MCAN_MSG_RAM_BASE_ADDR + offset) = data
#define READ_MSG_RAM(offset) *(vuint32_t*)(MCAN_MSG_RAM_BASE_ADDR + offset)
#define READ_MSG_RAM_8bit(offset) *(vuint8_t*)(MCAN_MSG_RAM_BASE_ADDR + offset)

#define BRS 0

// SIU macros
#define APC 0x2000
#define OBE 0x0200
#define IBE 0x0100
#define ODE 0x0020
#define SRC 0x0004
#define WPE 0x0002
#define WPS 0x0001
#define ALT0 0x0000
#define ALT1 0x0400
#define ALT2 0x0800
#define ALT3 0x0C00

#define ETPUA30_pin 144 // PCR number
#define ETPUA31_pin 145

#define LED1_pin ETPUA30_pin
#define LED2_pin ETPUA31_pin

/*******************************************************************************
* Local types
*******************************************************************************/

/*******************************************************************************
* Local function prototypes
*******************************************************************************/
static void HW_init(void);
static void SIU_Init(void);
static void PIT0_Init(void);
static void Terminal_ECHO(void);

static void MCAN_0_Init(void);
static void MCAN_1_Init(void);
static void MCAN_0_TransmitMsg(uint8_t ext, uint32_t ID, uint8_t length);
static void MCAN_1_Read_RXFIFO(uint8_t fifo);
static void MCAN_MSG_RAM_Init(void);
void MCAN_ISR(void);


/*******************************************************************************
* Local variables
*******************************************************************************/

/*******************************************************************************
* Local functions
*******************************************************************************/

/*******************************************************************************
Function Name : HW_init
Engineer      : David Tosenovjan
Date          : Aug-26-2015
Parameters    : 
Modifies      : 
Returns       : 
Notes         : initialization of the hw for the purposes of this example
Issues        : 
*******************************************************************************/
static void HW_init(void)
{
    /* set round robin for all slaves */
    XBAR.SGPCR0.B.ARB = 1;
    XBAR.SGPCR1.B.ARB = 1;
    XBAR.SGPCR2.B.ARB = 1;
    XBAR.SGPCR3.B.ARB = 1;
    XBAR.SGPCR4.B.ARB = 1;
    XBAR.SGPCR6.B.ARB = 1;
    XBAR.SGPCR7.B.ARB = 1;
    
    /* initialize GPIOs */
    SIU_Init();
}


/*******************************************************************************
Function Name : SIU_Init
Engineer      : David Tosenovjan
Date          : Aug-26-2015
Parameters    : 
Modifies      : 
Returns       : 
Notes         : 
Issues        : 
*******************************************************************************/
static void SIU_Init(void)
{
    SIU.PCR[LED1_pin].R = ALT0 | OBE;    // general purpose output
    SIU.PCR[LED2_pin].R = ALT0 | OBE;    // general purpose output
}


/*******************************************************************************
Function Name : PIT0_Init
Engineer      : Petr Stancik
Date          : Jul-22-2016
Parameters    : 
Modifies      : 
Returns       : 
Notes         : set for 1sec delay assuming 96MHz peripheral clock
Issues        : 
*******************************************************************************/
static void PIT0_Init(void)
{
    /* 30: MDIS = 0 to enable clock for PITs. */
    /* 31: FRZ = 1 for Timers stopped in debug mode */
    PIT.MCR.R = 0x00000001;    
    PIT.CH[0].LDVAL.R = 96000000 - 1;
        
    /* clear the TIF flag */
    PIT.CH[0].TFLG.R = 0x00000001;
        
    /* 31: TEN = 1 for timer active */
    PIT.CH[0].TCTRL.R = 0x00000001;
}


/*******************************************************************************
Function Name : Terminal_ECHO
Engineer      : David Tosenovjan
Date          : Aug-26-2015
Parameters    : 
Modifies      : 
Returns       : 
Notes         :                          
Issues        : 
*******************************************************************************/
static void Terminal_ECHO(void)
{
    char ch;
    
    if (ESCI_A.SR.B.RDRF == 0)
        return;
    else 
    {
        ESCI_A.SR.R = 0x20000000; // clear RDRF flag
        ch =(unsigned char) ESCI_A.DR.B.D; // read byte of Data
        switch(ch)
        {
            // enter
            case '\n':
            case '\r':
                printf("\r\n");
                break;
                    
            // other symbols
            default:
                printf("%c",ch); 
                fflush(stdout);
        } 
    } 
}

/*******************************************************************************
Function Name : MCAN_MSG_RAM_Init
Engineer      : Petr Stancik
Date          : Sep-15-2016
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Initialize shared MSG RAM
Issues        : NONE
*******************************************************************************/
static void MCAN_MSG_RAM_Init(void)
{
    int i;
    uint32_t *pMsgRAM = (uint32_t*)MCAN_MSG_RAM_BASE_ADDR;
    
    for(i=0;i<MCAN_MSG_RAM_LENGTH/4;i++)
    {
	*pMsgRAM++ =  0x0;
    }
}

/*******************************************************************************
Function Name : MCAN_0_Init
Engineer      : Petr Stancik
Date          : Sep-15-2016
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : MCAN_0 initialization
Issues        : NONE
*******************************************************************************/
static void MCAN_0_Init(void)
{
    // put module to Init mode
    MCAN_0.CCCR.B.INIT = 0x1;
    while(MCAN_0.CCCR.B.INIT == 0)
    {
	asm("nop");
    }

    // enable CCE bit to change config
    MCAN_0.CCCR.B.CCE = 0x1;
    while(MCAN_0.CCCR.B.CCE==0)
    {
	asm("nop");
    }

#if BRS
    MCAN_0.CCCR.B.CMR = 0x2;	// enable CAN FD with bit rate switching
    MCAN_0.CCCR.B.CME = 0x2;

    MCAN_0.BTP.R = 0x00011E77;  // Set time quanta for 0.5Mbps SYNC=1, TSEG1=30+1, TSEG2=7+1, SJW=7+1

    MCAN_0.FBTP.R = 0x03810D32;	 // 0x03810D32 time quanta for FD for 1Mbps SYNC=1, TSEG1=14+1, TSEG2=3+1, SJW=2+1
                                 // 0x03800522 time quanta for FD for 4Mbps SYNC=1, TSEG1=5+1, TSEG2=2+1, SJW=2+1
                                 // 0x03800201 time quanta for FD for 8Mbps SYNC=1, TSEG1=2+1, TSEG2=0+1, SJW=1+1
                                 // delay compensation set to 4, vary if fault occurs at high speed
#else
    MCAN_0.CCCR.B.CMR = 0x1;	// enable CAN FD without bit rate switching
    MCAN_0.CCCR.B.CME = 0x1;

    MCAN_0.BTP.R = 0x00011E77;  // Set time quanta for 0.5Mbps SYNC=1, TSEG1=30+1, TSEG2=7+1, SJW=7+1
#endif
    
    MCAN_0.GFC.B.ANFS = 3;	// Reject non-matching standard frames in Rx FIFO 0
    MCAN_0.GFC.B.ANFE = 3;	// Reject non-matching extended frames in Rx FIFO 1
    MCAN_0.GFC.B.RRFS = 1;	// Reject Remote Frames Standard
    MCAN_0.GFC.B.RRFE = 1;	// Reject Remote Frames Extended
    
  
    MCAN_0.TXBC.R = (0				// tx buffer
	               | 2 << 16		// space for 2 messages
		       | MCAN0_TXBC_TBSA); 	// starting at offset MCAN0_TXBC_TBSA

    MCAN_0.RXESC.R = 0x00000777;	//set for 64 data bytes both RX FIFO 0 and FIFO 1
    MCAN_0.TXESC.R = 0x00000007;	//set for 64 data bytes Tx buffer

    asm("nop");
    asm("nop");


    MCAN_0.CCCR.B.CCE = 0x0;
    MCAN_0.CCCR.B.INIT = 0x0;				//re-enable CCCR protection
    while((MCAN_0.CCCR.R & 0x1)==1)
    {
	asm("nop");
    }

    
    /* configure MCAN0TX and MCAN0RX pin functions on PCR83 and PCR84 */
    SIU.PCR[83].R = 0x0E0C; /* MCAN0TX, push/pull, max slew rate */
    SIU.PCR[84].R = 0x0D03; /* MCAN0RX, weak pull device disabled */
    SIU.IMUX[1].R = 1;  // select MCANRX0 for PCR[84]
    
}

/*******************************************************************************
Function Name : MCAN_1_Init
Engineer      : Petr Stancik
Date          : Sep-15-2016
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : MCAN_1 initialization
Issues        : NONE
*******************************************************************************/
static void MCAN_1_Init(void)
{

    // put module to Init mode
    MCAN_1.CCCR.B.INIT = 0x1;
    while(MCAN_1.CCCR.B.INIT == 0)
    {
	asm("nop");
    }

    // enable CCE bit to change config
    MCAN_1.CCCR.B.CCE = 0x1;
    while(MCAN_1.CCCR.B.CCE==0)
    {
	asm("nop");
    }

#if BRS
    MCAN_1.CCCR.B.CMR = 0x2;	// enable CAN FD with bit rate switching
    MCAN_1.CCCR.B.CME = 0x2;

    MCAN_1.BTP.R = 0x00011E77;  // Set time quanta for 0.5Mbps SYNC=1, TSEG1=30+1, TSEG2=7+1, SJW=7+1

    MCAN_1.FBTP.R = 0x03810D32;	 // 0x03810D32 time quanta for FD for 1Mbps SYNC=1, TSEG1=14+1, TSEG2=3+1, SJW=2+1
                                 // 0x03800522 time quanta for FD for 4Mbps SYNC=1, TSEG1=5+1, TSEG2=2+1, SJW=2+1
                                 // 0x03800201 time quanta for FD for 8Mbps SYNC=1, TSEG1=2+1, TSEG2=0+1, SJW=1+1
                          	// delay compensation set to 4, vary if fault occurs at high speed
#else
    MCAN_1.CCCR.B.CMR = 0x1;	// enable CAN FD without bit rate switching
    MCAN_1.CCCR.B.CME = 0x1;

    MCAN_1.BTP.R = 0x00011E77;  // Set time quanta for 0.5Mbps SYNC=1, TSEG1=30+1, TSEG2=7+1, SJW=7+1
#endif
    
    MCAN_1.GFC.B.ANFS = 0;	// Accept non-matching standard frames in Rx FIFO 0
    MCAN_1.GFC.B.ANFE = 1;	// Accept non-matching extended frames in Rx FIFO 1
    MCAN_1.GFC.B.RRFS = 1;	// Reject Remote Frames Standard
    MCAN_1.GFC.B.RRFE = 1;	// Reject Remote Frames Extended
    
    MCAN_1.SIDFC.R = (0
	               | 2 << 16		// 2 standard message ID filter elements
		       | MCAN1_SIDFC_FLSSA);	// filter list starting from offset MCAN1_SIDFC_FLSSA 

    MCAN_1.XIDFC.R = (0
	               | 2 << 16		// 2 Extended ID's filter used in this network
		       | MCAN1_XIDFC_FLESA);    // filter list starting from offset MCAN1_XIDFC_FLESA

    MCAN_1.RXF0C.R = (0
	               | 1 << 31		// FIFO0 overwrite
		       | 2 << 16		// hold 2 messages
		       | MCAN1_RXF0C_F0SA); 	// starting at offset MCAN1_RXF0C_F0SA
	    
    MCAN_1.RXF1C.R = (0
	               | 1 << 31		// FIFO1 overwrite
		       | 2 << 16		// hold 2 messages
		       | MCAN1_RXF1C_F1SA); 	// starting at offset MCAN1_RXF1C_F1SA
    
    MCAN_1.RXESC.R = 0x00000777;	//set for 64 data bytes both RX FIFO 0 and FIFO 1
    MCAN_1.TXESC.R = 0x00000007;	//set for 64 data bytes Tx buffer

    MCAN_1.IE.B.RF0NE = 1;	// Rx FIFO 0 New Message Interrupt Enable 
    MCAN_1.IE.B.RF1NE = 1;	// Rx FIFO 1 New Message Interrupt Enable
    MCAN_1.ILE.B.EINT0 = 1;	// M_CAN interrupt line 0 enabled

    asm("nop");
    asm("nop");


    MCAN_1.CCCR.B.CCE = 0x0;
    MCAN_1.CCCR.B.INIT = 0x0;				//re-enable CCCR protection
    while((MCAN_1.CCCR.B.INIT & 0x1)==1)
    {
	asm("nop");
    }
     
    /* configure MCAN1TX and MCAN1RX pin functions on PCR246 and PCR247 */
    SIU.PCR[246].R = 0x0E0C; /* MCAN1TX, push/pull, max slew rate */
    SIU.PCR[247].R = 0x0D03; /* MCAN1RX, weak pull device disabled */
    SIU.IMUX[1].R |= 4;  // select MCANRX1 for PCR[247]
    
}

/*******************************************************************************
Function Name : MCAN_ID_init
Engineer      : Petr Stancik
Date          : Sep-15-2016
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : MCAN_1 ID filter tables initialization
Issues        : NONE
*******************************************************************************/
void MCAN_ID_init()
{
 
    //MCAN1 std ID table MCAN1_SIDFC_FLSSA
    WRITE_MSG_RAM( MCAN1_SIDFC_FLSSA,
	           0		// Standard Message ID filter element 1
	            | 0x2<<30	// Classic filter: SFID1 = filter (ID), SFID2 = mask
		    | 0x1<<27	// Store in Rx FIFO 0 if filter matches
	            | 0x123<<16	// ID = 0x123
		    | 0x7FF); 	// mask = 0x7FF 
 

    WRITE_MSG_RAM( MCAN1_SIDFC_FLSSA + 0x04,
	           0		// Standard Message ID filter element 2
	            | 0x2<<30	// Classic filter: SFID1 = filter (ID), SFID2 = mask
		    | 0x1<<27	// Store in Rx FIFO 0 if filter matches
		    | 0x456<<16	// ID = 0x456
		    | 0x7FF); 	// mask = 0x7FF

    //MCAN1 ext ID table at offset MCAN1_XIDFC_FLESA
    WRITE_MSG_RAM( MCAN1_XIDFC_FLESA,
	           0		// extended Message ID filter element 1, word F0
	            | 0x2<<29	// Store in Rx FIFO 1 if filter matches 
		    | 0x123);	// ID = 0x123

    WRITE_MSG_RAM( MCAN1_XIDFC_FLESA + 0x04,
	           0		// extended Message ID filter element 1, word F1
	            | 0x2<<30	// Classic filter: EFID1 = filter (ID), EFID2 = mask
		    | 0x1FFFFFFF);  // mask = 0x1FFFFFFF

    WRITE_MSG_RAM( MCAN1_XIDFC_FLESA + 0x08,
	           0		// extended Message ID filter element 2, word F0
	            | 0x2<<29	// Store in Rx FIFO 1 if filter matches 
		    | 0x456);	// ID = 0x123

    WRITE_MSG_RAM( MCAN1_XIDFC_FLESA + 0x0C,
	           0		// extended Message ID filter element 2, word F1
	            | 0x2<<30	// Classic filter: EFID1 = filter (ID), EFID2 = mask
		    | 0x1FFFFFFF);  // mask = 0x1FFFFFFF
}


/*******************************************************************************
Function Name : MCAN_0_TransmitMsg
Engineer      : Petr Stancik
Date          : Sep-15-2016
Parameters    : ext ... 0-standard ID, 1-extended ID
                ID ... ID to be sent
                length ... number of bytes to be sent, max 8 as FD is not used
Modifies      : NONE
Returns       : NONE
Notes         : MCAN_0 simple message transmit, 
Issues        : NONE
*******************************************************************************/
static void MCAN_0_TransmitMsg(uint8_t ext, uint32_t ID, uint8_t length) 
{
    uint32_t txID;
    uint8_t dlc = 0;

    if (ext==0) txID = ID<<18;
    else txID = ID;
    
    if (MCAN_0.TXBRP.R == 0)
    {

	int z;

	/*calculate dlc value from no. bytes*/
	if(length < 0x09)
	{
	    dlc = length;
	}else if(length < 0x0D){
	    dlc = 0x9;
	}else if(length < 0x11){
	    dlc = 0xA;
	}else if(length < 0x15){
	    dlc = 0xB;
	}else if(length < 0x19){
	    dlc = 0xC;
	}else if(length < 0x21){
	    dlc = 0xD;
	}else if(length < 0x31){
	    dlc = 0xE;
	}else{
	    dlc = 0xF;
	}

	WRITE_MSG_RAM( MCAN0_TXBC_TBSA,
		       0		// Tx Buffer 0 T0 word
		        | ext<<30	// standard ID
			| 0<<29		// transmit data frame
			| txID); 	// message ID

	WRITE_MSG_RAM( MCAN0_TXBC_TBSA  + 4,
		       0		// Tx Buffer 0 T1 word
		        | 0<<23		// do not store TX event
			| (dlc << 16)); // dlc... data length

	WRITE_MSG_RAM( MCAN0_TXBC_TBSA  + 8 , MCAN_0_data[0] | (1)<<8 | (2)<<16 | (3)<<24);	// Tx Buffer 0 T3 word
	
	for(z = 4; z < length; z+=4)	// fill rest of Tx Buffer 0 Tn words with data
	{	
	    WRITE_MSG_RAM( MCAN0_TXBC_TBSA  + 8 + z, z | (z+1)<<8 | (z+2)<<16 | (z+3)<<24);
	}
	
	MCAN_0.TXBAR.R = 0x1;	// start transmission for M_CAN0 TX buffer 0
   }
}

/*******************************************************************************
Function Name : MCAN_1_Read_RXFIFO
Engineer      : Petr Stancik
Date          : Sep-15-2016
Parameters    : fifo .. RX FIFO number
Modifies      : NONE
Returns       : NONE
Notes         : Read message from MCAN_1 RX FIFO
Issues        : NONE
*******************************************************************************/
static void MCAN_1_Read_RXFIFO(uint8_t fifo) 
{
  uint32_t temp = 0;
  uint8_t dlc,length,i=0,j=3;
  uint8_t get_index;  // index of FIFO element to be read
  uint16_t fifo_item_offset ; // real offset in msg RAM, 2 word + 8 bytes of data
  uint8_t *pRXdata = &MCAN_1_data[0];

  if(fifo)
  {
      get_index = MCAN_1.RXF1S.B.F1GI; // index of FIFO element to be read
      fifo_item_offset = MCAN1_RXF1C_F1SA + 72 * get_index; // real offset in msg RAM, 2 word + 64 bytes of data
  }
  else
  {
      get_index = MCAN_1.RXF0S.B.F0GI; // index of FIFO element to be read
      fifo_item_offset = MCAN1_RXF0C_F0SA + 72 * get_index; // real offset in msg RAM, 2 word + 64 bytes of data
  }
  
  // read message word R0 from RXFIFO 0
  temp = READ_MSG_RAM( fifo_item_offset + 0);
  // read message word R1 from RXFIFO 0
  temp = READ_MSG_RAM( fifo_item_offset + 4);

  dlc = (temp&0x000F0000)>>16; // get number of received bytes

  /*calculate no. bytes from dlc value*/
  if(dlc < 0x09)
  {
      length = dlc;
  }else if(dlc == 0x9){
      length = 12;
  }else if(dlc == 0xA){
      length = 16;
  }else if(dlc == 0xB){
      length = 20;
  }else if(dlc == 0xC){
      length = 24;
  }else if(dlc == 0xD){
      length = 32;
  }else if(dlc == 0xE){
      length = 48;
  }else{
      length = 64;
  }

  while(length--)	// read dlc bytes from FIFO
  {
      *pRXdata++ = READ_MSG_RAM_8bit( fifo_item_offset + 8 + i + j);
      if(j==0) j=4,i+=4;
      j--;
  }

  if(fifo)  MCAN_1.RXF1A.R = get_index; // acknowledge the reading of RXFIFO "get_index" element
  else MCAN_1.RXF0A.R = get_index; // acknowledge the reading of RXFIFO "get_index" element

  /* do someting with the received data */
  /* drive the output pin with the received data */
  SIU.GPDO[LED1_pin].R = MCAN_1_data[0]; 
}

/*******************************************************************************
Function Name : MCAN_ISR
Engineer      : Petr Stancik
Date          : Sep-15-2016
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : MCAN combined Interrupt routine. Read message from MCAN_1 RX FIFO
Issues        : NONE
*******************************************************************************/
void MCAN_ISR(void)
{
    if(MCAN_1.IR.B.RF0N == 1) 	// is there new message in RXFIFO_0
    {
	MCAN_1_Read_RXFIFO(0);	// read RXFIFO_0
	MCAN_1.IR.R = 1;	// clear RF0N interrupt flag

    }

    if(MCAN_1.IR.B.RF1N == 1) 	// is there new message in RXFIFO_1
    {
	MCAN_1_Read_RXFIFO(1);	// read RXFIFO_1
	MCAN_1.IR.R = 0x10;	// clear RF1N interrupt flag

    }
}    
/*******************************************************************************
* Global functions
*******************************************************************************/

/*******************************************************************************
Function Name : main
Engineer      : Petr Stancik
Date          : Sep-15-2016
Parameters    : 
Modifies      : 
Returns       : 
Notes         :                             
Issues        : 
*******************************************************************************/
void main (void)
{    
    vint32_t counter = 0, i;
    
    /* Hardware initialization */
    HW_init();

    PIT0_Init();	// set to generate 1sec interval

    MCAN_MSG_RAM_Init();

    MCAN_0_Init();
    MCAN_1_Init();
    MCAN_ID_init();							   /* Initialise MCAN ID's*/
    
    Interrupts_init();
    Interrupts_enable_Core0();

   // printf("MPC5777C FlexCAN demo\r\n");
   // printf("\n\r");
    
    /* Start the second core */ 
    SIU.RSTVEC1.R = (unsigned long)__start | 0x00000001; // VLE
    //SIU.RSTVEC1.R = (unsigned long)__start; // Book-E

    /* Loop forever */
    while (1) 
    {
	if(PIT.CH[0].TFLG.B.TIF)
    	{
	     
	    MCAN_0_data[0]=tx_cnt1++;

	    switch (tx_cnt1&3) {
		case 0:
		        MCAN_0_TransmitMsg(0,0x123, 2);	// Transmit one message from MCAN_0 TX buffer 0, std ID =0x123, data length 2
		break;
		case 1:
		        MCAN_0_TransmitMsg(0,0x456, 16); // Transmit one message from MCAN_0 TX buffer 0, std ID =0x456, data length 16 
		break;
		case 2:
		        MCAN_0_TransmitMsg(1,0x123, 22); // Transmit one message from MCAN_0 TX buffer 0, ext ID =0x456, data length 22 
		break;
		case 3:
		        MCAN_0_TransmitMsg(1,0x456, 8); // Transmit one message from MCAN_0 TX buffer 0, ext ID =0x456, data length 8
		break;
		default:
		break;
	    }
		    
	    PIT.CH[0].TFLG.B.TIF = 1;     // Clear PIT_0 interrupt flag
  	}

	

	i++;
#if 0
	/* delay */
        while(counter < 500000)
        {
            counter++;
            Terminal_ECHO();

        }  
        counter = 0;      
#endif
    }
    
}
